{% load i18n fb_versions %}
{% ifequal value.filetype "Image" %}
{% version_object value.path ADMIN_THUMBNAIL as thumbnail_version %}
{% if thumbnail_version %}
<p class="preview">
	<a href="{{ value.url }}" target="_blank">
        <img src="{{ thumbnail_version.url }}" class="preview" />
    </a>
</p>
{% else %}
<ul class="errorlist"><li>{% trans "File not found" %}</li></ul>
{% endif %}
{% endifequal %}