from filebrowser.tests.test_settings import SettingsTests
from filebrowser.tests.test_base import FileObjectPathTests, FileObjectUnicodeTests
from filebrowser.tests.test_sites import *

# These tests will create directories and files within MEDIA_ROOT
# Only use this with your local environment
from filebrowser.tests.test_base import FileObjectAttributeTests, FileListingTests
from filebrowser.tests.test_versions import VersionTemplateTagsTests
from filebrowser.tests.test_commands import CommandsTests
